#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['vickers_hardness', 'vickers_hardness.data', 'vickers_hardness.utils']

package_data = \
{'': ['*'], 'vickers_hardness': ['examples/*', 'tests/*']}

setup(name='vickers_hardness',
      version='0.1.9',
      description='Predicting load dependent Vickers hardness based on chemical composition.',
      author='Ziyan Zhang',
      author_email='"Sterling G. Baird" <sterling.baird@utah.edu>',
      url=None,
      packages=packages,
      package_data=package_data,
     )
